# Copyright (C) 2007, 2008, 2009 by Intevation GmbH
# Authors:
# Bernhard Herzog <bh@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

"""Tests for the Status classes"""

import os
import stat
import unittest
from datetime import datetime

from treepkg.status import RevisionStatus, Status, EnumFieldDesc, \
     StringFieldDesc, DateFieldDesc
from treepkg.util import writefile

from filesupport import FileTestMixin



class TestStatus(unittest.TestCase, FileTestMixin):

    def setUp(self):
        self.filename = self.temp_file_name("status")
        if os.path.exists(self.filename):
            os.remove(self.filename)

    def test_status(self):
        status = RevisionStatus(self.filename)
        status.error()

        otherstatus = RevisionStatus(self.filename)
        self.assertEquals(otherstatus.status.name, "error")

    def test_status_file_permissions(self):
        status = RevisionStatus(self.filename)
        status.error()

        mode = os.stat(self.filename).st_mode
        self.assertEquals(stat.S_IMODE(mode) & 0444, 0444)

    def test_getting_unknown_fields(self):
        status = RevisionStatus(self.filename)
        self.assertRaises(AttributeError, getattr, status, "unknown_field")

    def test_setting_unknown_fields(self):
        status = RevisionStatus(self.filename)
        self.assertRaises(AttributeError,
                          setattr, status, "unknown_field", "some value")

    def test_default_values(self):
        status = RevisionStatus(self.filename)
        self.assertEquals(status.status.name, "unknown")
        self.assertEquals(status.start, None)
        self.assertEquals(status.stop, None)
        self.assertEquals(status.tags, "")

    def test_date(self):
        timestamp = datetime(2007, 3, 9, 17, 32, 55)
        status = RevisionStatus(self.filename)
        status.start = timestamp

        otherstatus = RevisionStatus(self.filename)
        self.assertEquals(otherstatus.start, timestamp)

    def test_magic(self):
        writefile(self.filename,
                  "Some other magic\nstart: 2007-03-09 17:32:55\n")
        self.assertRaises(ValueError, RevisionStatus, self.filename)

    def test_status_enum(self):

        class TestStatus(Status):
            status = EnumFieldDesc()
            status.add("not_running", "Process is not running",
                       default=True)
            status.add("running", "Process is running")
            status.add("error", "An error occurred")

        status = TestStatus(self.filename)
        status.running()

        otherstatus = TestStatus(self.filename)
        self.assertEquals(otherstatus.status.name, "running")

    def test_serialize(self):
        class TestStatus(Status):
            _magic = "TestStatus 1.0\n"

            status = EnumFieldDesc()
            status.add("not_running", "Process is not running",
                       default=True)
            status.add("running", "Process is running")
            status.add("error", "An error occurred")

            start = DateFieldDesc(default=None)

            name = StringFieldDesc(default="fred")

        status = TestStatus(None)
        status.running()
        status.start = datetime(2007, 3, 9, 17, 32, 55)
        status.name = "Dave"

        self.assertEquals(sorted(status.serialize()),
                          ["TestStatus 1.0\n",
                           "name: Dave\n",
                           "start: 2007-03-09 17:32:55\n",
                           "status: running\n"])

    def test_deserialize(self):
        class TestStatus(Status):
            _magic = "TestStatus 1.0\n"

            status = EnumFieldDesc()
            status.add("not_running", "Process is not running",
                       default=True)
            status.add("running", "Process is running")
            status.add("error", "An error occurred")

            start = DateFieldDesc(default=None)

            name = StringFieldDesc(default="fred")

        status = TestStatus(None)
        status.deserialize(iter(["TestStatus 1.0\n",
                                 "name: Dave\n",
                                 "start: 2007-03-09 17:32:55\n",
                                 "status: running\n"]))
        self.assertEquals(status.status.name, "running")
        self.assertEquals(status.start, datetime(2007, 3, 9, 17, 32, 55))
        self.assertEquals(status.name, "Dave")
